import requests
import re
from pathlib import Path
import time
import json
from qualys_etl.etld_lib import etld_lib_credentials as etld_lib_credentials
from qualys_etl.etld_lib import etld_lib_config as etld_lib_config
from qualys_etl.etld_lib import etld_lib_functions as etld_lib_functions
from qualys_etl.etld_lib import etld_lib_extract_transform_load_distribute as etld_lib_extract_transform_load_distribute

global json_file
global url
global multi_proc_host_ids
global multi_proc_batch_number
global json_file_utc_run_datetime
global qualys_headers_multi_proc_dict


def asset_inventory_extract(asset_last_updated, last_seen_assetid, utc_datetime, batch_number, proc_dict):
    global json_file
    global url
    global multi_proc_host_ids
    global multi_proc_batch_number
    global json_file_utc_run_datetime
    global qualys_headers_multi_proc_dict

    qualys_headers_multi_proc_dict = proc_dict
    json_file_utc_run_datetime = utc_datetime
    multi_proc_batch_number = batch_number
    cred_dict = etld_lib_credentials.get_cred()
    bearer = etld_lib_credentials.get_bearer(update_bearer=False)
    url = f"https://{cred_dict['gateway_fqdn_server']}/rest/2.0/search/am/asset"
    # /rest/2.0/search/am/asset?assetLastUpdated=2021-06-01T00:00:00Z&lastSeenAssetId=0
    if isinstance(etld_lib_config.asset_inventory_multi_proc_batch_size, str):
        page_size = 300
    elif etld_lib_config.asset_inventory_multi_proc_batch_size > 300:
        page_size = 300
    elif etld_lib_config.asset_inventory_multi_proc_batch_size < 1:
        page_size = 300
    else:
        page_size = etld_lib_config.asset_inventory_multi_proc_batch_size

    url = f"{url}?assetLastUpdated={asset_last_updated}&lastSeenAssetId={last_seen_assetid}&pageSize={page_size}"

    # payload = {'assetLastUpdated': asset_last_updated,
    #            'lastSeenAssetId': last_seen_assetid}

    headers = {'X-Requested-With': 'qualysetl', 'Authorization': bearer, 'Content-Type': 'application/json'}

#    print_payload = payload.copy()
    etld_lib_functions.logger.info(f"api call     - {url}")

    json_file_prefix = re.sub(".json$", "", str(etld_lib_config.asset_inventory_json_file))
    json_batch_assetLastUpdated = re.sub(':', '_', f"{asset_last_updated}")
    json_file_utc_run_datetime_filename = re.sub(':', '_', f"{json_file_utc_run_datetime}")
    json_file_name = f'{Path(json_file_prefix).name}_utc_run_datetime_{json_file_utc_run_datetime_filename}' \
                     f'_utc_assetLastUpdated_{json_batch_assetLastUpdated}' \
                     f'_{multi_proc_batch_number}.json'
    json_file = Path(etld_lib_config.asset_inventory_json_dir, json_file_name)

    chunk_size_calc = 20480
    try_extract_max_count = 15
    http_conn_timeout = 300
    etld_lib_extract_transform_load_distribute.extract_qualys(
        try_extract_max_count=try_extract_max_count,
        url=url,
        headers=headers,
        payload={},
        http_conn_timeout=http_conn_timeout,
        chunk_size_calc=chunk_size_calc,
        output_file=json_file,
        cred_dict=cred_dict,
        qualys_headers_dict=qualys_headers_multi_proc_dict,
        multi_proc_batch_number=multi_proc_batch_number,
        extract_validation_type='json')


def asset_inventory_extract_count(asset_last_updated, last_seen_assetid, utc_datetime, batch_number, proc_dict):
    global json_file
    global url
    global multi_proc_host_ids
    global multi_proc_batch_number
    global json_file_utc_run_datetime
    global qualys_headers_multi_proc_dict

    qualys_headers_multi_proc_dict = proc_dict
    json_file_utc_run_datetime = utc_datetime
    multi_proc_batch_number = batch_number
    cred_dict = etld_lib_credentials.get_cred()
    bearer = etld_lib_credentials.get_bearer(update_bearer=False)
    # rest/2.0/count/am/asset
    url = f"https://{cred_dict['gateway_fqdn_server']}/rest/2.0/count/am/asset"
    # /rest/2.0/search/am/asset?assetLastUpdated=2021-06-01T00:00:00Z&lastSeenAssetId=0
    if isinstance(etld_lib_config.asset_inventory_multi_proc_batch_size, str):
        page_size = 300
    elif etld_lib_config.asset_inventory_multi_proc_batch_size > 300:
        page_size = 300
    elif etld_lib_config.asset_inventory_multi_proc_batch_size < 1:
        page_size = 300
    else:
        page_size = etld_lib_config.asset_inventory_multi_proc_batch_size

    url = f"{url}?assetLastUpdated={asset_last_updated}&lastSeenAssetId={last_seen_assetid}"

    # payload = {'assetLastUpdated': asset_last_updated,
    #            'lastSeenAssetId': last_seen_assetid}

    headers = {'X-Requested-With': 'qualysetl', 'Authorization': bearer, 'Content-Type': 'application/json'}

#    print_payload = payload.copy()
    etld_lib_functions.logger.info(f"api call     - {url}")

    json_file_prefix = "asset_inventory_extract_count"
    json_batch_assetLastUpdated = re.sub(':', '_', f"{asset_last_updated}")
    json_file_utc_run_datetime_filename = re.sub(':', '_', f"{json_file_utc_run_datetime}")
    json_file_name = f'{json_file_prefix}_utc_run_datetime_{json_file_utc_run_datetime_filename}' \
                     f'_utc_assetLastUpdated_{json_batch_assetLastUpdated}' \
                     f'.json'
    json_file = Path(etld_lib_config.asset_inventory_json_dir, json_file_name)

    chunk_size_calc = 20480
    try_extract_max_count = 5
    http_conn_timeout = 300
    etld_lib_extract_transform_load_distribute.extract_qualys(
        try_extract_max_count=try_extract_max_count,
        url=url,
        headers=headers,
        payload={},
        http_conn_timeout=http_conn_timeout,
        chunk_size_calc=chunk_size_calc,
        output_file=json_file,
        cred_dict=cred_dict,
        qualys_headers_dict=qualys_headers_multi_proc_dict,
        multi_proc_batch_number=multi_proc_batch_number,
        extract_validation_type='json')

    asset_inventory_log_count()


def asset_inventory_log_count():
    try:
        with open(str(json_file), "r", encoding='utf-8') as read_file:
            ai_count = json.load(read_file)
            if "responseCode" in ai_count.keys():
                if ai_count['responseCode'] == 'SUCCESS':
                    etld_lib_functions.logger.info(f"Asset Inventory Count: {ai_count['count']}")
                else:
                    raise Exception(f"Asset Inventory Count Failed, responseCode: {ai_count['responseCode']},"
                                    f" responseMessage: {ai_count['responseMessage']}")
    except Exception as e:
        etld_lib_functions.logger.error(f"Exception: {e}")
        etld_lib_functions.logger.error(f"Potential JSON File corruption or api error detected: {json_file}")
        exit(1)


# def get_qualys_limits_from_asset_inventory():
#     global json_file
#     global url
#     global multi_proc_host_ids
#     global multi_proc_batch_number
#     global json_file_utc_run_datetime
#     global qualys_headers_multi_proc_dict
#     etld_lib_credentials.main()
#     cred_dict = etld_lib_credentials.get_cred()
#     authorization = cred_dict['authorization']  # Base64 user:password
#     use_cookie = etld_lib_credentials.use_cookie  # If true, use cookie auth
#     url = f"https://{cred_dict['api_fqdn_server']}/api/2.0/fo/asset/host/vm/detection/"
#
#     payload = {'action': 'list',
#                'truncation_limit': '1',
#                }
#
#     if use_cookie is False:
#         headers = {'X-Requested-With': 'qualysetl', 'Authorization': authorization}
#     else:
#         headers = {'X-Requested-With': 'qualysetl', 'Cookie': etld_lib_credentials.cookie_file()}
#
#     json_file = Path("/dev/null")
#     chunk_size_calc = 20480
#     try_extract_max_count = 5
#     http_conn_timeout = 30
#     multi_proc_batch_number = "check_headers"
#     etld_lib_extract_transform_load_distribute.extract_qualys(
#         try_extract_max_count=try_extract_max_count,
#         url=url,
#         headers=headers,
#         payload=payload,
#         http_conn_timeout=http_conn_timeout,
#         chunk_size_calc=chunk_size_calc,
#         output_file=json_file,
#         cred_dict=cred_dict,
#         qualys_headers_dict=qualys_headers_multi_proc_dict,
#         multi_proc_batch_number=multi_proc_batch_number,
#         extract_validation_type='json')


def start_msg_asset_inventory_extract():
    etld_lib_functions.logger.info(f"start")


def end_msg_asset_inventory_extract():
    global url
    global json_file
    global json_file_utc_run_datetime
    etld_lib_functions.log_file_info(url, 'in')
    etld_lib_functions.logger.info(f"Run Date: {json_file_utc_run_datetime}")
    etld_lib_functions.log_file_info(json_file)
    etld_lib_functions.logger.info(f"end")


def main(args=None):
    start_msg_asset_inventory_extract()
    etld_lib_credentials.main()
    asset_inventory_extract()
    end_msg_asset_inventory_extract()


if __name__ == "__main__":
    etld_lib_functions.main(my_logger_prog_name='asset_inventory_extract')
    etld_lib_config.main()
    etld_lib_credentials.main()
    main()






import requests
import re
import time
from pathlib import Path
from qualys_etl.etld_lib import etld_lib_credentials as etld_lib_credentials
from qualys_etl.etld_lib import etld_lib_config as etld_lib_config
from qualys_etl.etld_lib import etld_lib_functions as etld_lib_functions
from qualys_etl.etld_lib import etld_lib_extract_transform_load_distribute as etld_lib_extract_transform_load_distribute
from qualys_etl.etld_lib import etld_lib_sqlite_tables as etld_lib_sqlite_tables
global xml_files
global url
global payload_list
global qualys_headers
global host_list_vm_processed_after
global xml_file


def remove_old_files():
    try:
        if Path(etld_lib_config.host_list_sqlite_file).is_file():
            etld_lib_functions.logger.info(f"Removing old sqlite file: {etld_lib_config.host_list_sqlite_file}")
            Path(etld_lib_config.host_list_sqlite_file).unlink()
        if Path(etld_lib_config.host_list_shelve_file).is_file():
            etld_lib_functions.logger.info(f"Removing old shelve file: {etld_lib_config.host_list_shelve_file}")
            Path(etld_lib_config.host_list_shelve_file).unlink()
        if Path(etld_lib_config.host_list_json_file).is_file():
            etld_lib_functions.logger.info(f"Removing old json file: {etld_lib_config.host_list_json_file}")
            Path(etld_lib_config.host_list_json_file).unlink()
        if Path(etld_lib_config.host_list_csv_file).is_file():
            etld_lib_functions.logger.info(f"Removing old csv file: {etld_lib_config.host_list_csv_file}")
            Path(etld_lib_config.host_list_csv_file).unlink()
    except Exception as e:
        etld_lib_functions.logger.error(f"{e}")
        exit(1)


def host_list_extract():
    global xml_files
    global url
    global payload_list
    global qualys_headers

    payload_list = []
    vm_processed_after = etld_lib_config.host_list_vm_processed_after  # UTC Date or 0 for all
    host_list_payload_option = etld_lib_config.host_list_payload_option  # notags or tags to used tag list
    if etld_lib_config.host_list_show_tags == '0':
        show_tags = '0'
    else:
        show_tags = '1'

    cred_dict = etld_lib_credentials.get_cred()
    authorization = cred_dict['authorization']  # Base64 user:password
    use_cookie = etld_lib_credentials.use_cookie  # If true, use cookie auth
    url = f"https://{cred_dict['api_fqdn_server']}/api/2.0/fo/asset/host/"  # Qualys Endpoint
    xml_files = {'host_list_other_xml_file': etld_lib_config.host_list_other_xml_file}

#    if host_list_payload_option == 'notags':
#        provider_list = ['notags']
#        xml_files = {'host_list_other_xml_file': etld_lib_config.host_list_other_xml_file }
#    else:
#        provider_list = ['ec2', 'gcp', 'azure', 'other']
#        xml_files = {'host_list_other_xml_file': etld_lib_config.host_list_other_xml_file,
#                     'host_list_ec2_xml_file': etld_lib_config.host_list_ec2_xml_file,
#                     'host_list_gcp_xml_file': etld_lib_config.host_list_gcp_xml_file,
#                     'host_list_azure_xml_file': etld_lib_config.host_list_azure_xml_file
#                     }


    # for provider in provider_list:
    #     if provider == 'notags':
    #         payload = {'action': 'list',
    #                    'details': 'All',
    #                    'truncation_limit': '0',
    #                    'show_tags': f"{show_tags}",
    #                    'show_asset_id': '1',
    #                    }
    #         xml_file = xml_files[f"host_list_other_xml_file"]
    #     else:
    #         xml_file = xml_files[f"host_list_{provider}_xml_file"]
    #         payload = {'action': 'list',
    #                    'details': 'All',
    #                    'use_tags': '1',
    #                    'truncation_limit': '0',
    #                    'tag_set_by': 'name',
    #                    'show_cloud_tags': '1',
    #                    'show_tags': f"{show_tags}",
    #                    'show_asset_id': '1',
    #                  }

        # if provider in ('ec2', 'gcp', 'azure'):
        #     payload['host_metadata'] = provider.replace('gcp', 'google')  # adjust if gcp to google for option
        #     payload['tag_set_include'] = 'qetl-all-' + provider   # ec2-all or gcp-all or azure-all
        # elif provider in 'notags':
        #     pass
        # elif provider in 'other':
        #     payload['tag_set_include'] = "qetl-all-hosts"
        #     payload['tag_set_exclude'] = "qetl-all-ec2,qetl-all-gcp,qetl-all-azure"

#        if vm_processed_after != "0":  # Set vm_processed_after to 0 get all assets scanned or un-scanned.
#            payload['vm_processed_after'] = vm_processed_after

    payload = {'action': 'list',
               'details': 'All',
               'truncation_limit': '0',
               'show_tags': f"{show_tags}",
               'show_cloud_tags': '1',
               'show_asset_id': '1',
               'host_metadata': 'all',
               }
    host_xml_file = xml_files[f"host_list_other_xml_file"]
    # provider = 'notags'
    if not str(vm_processed_after).__contains__("1970"):  # Set to 1970 to remove vm_processed_after and process alldata
        payload['vm_processed_after'] = vm_processed_after

    if use_cookie is False:
        headers = {'X-Requested-With': 'qualysetl', 'Authorization': authorization}
    else:
        headers = {'X-Requested-With': 'qualysetl', 'Cookie': etld_lib_credentials.get_cookie(update_cookie=True)}

    # etld_lib_functions.logger.info(f"provider     - {provider}")

    etld_lib_functions.logger.info(f"api call     - {url}")
    etld_lib_functions.logger.info(f"api options  - {payload}")
    etld_lib_functions.logger.info(f"api cookie   - {use_cookie}")

    payload_list.append(payload)
    # TODO: build payload_list in separate method from requests so options can be adjusted.

    chunk_size_calc = 20480
    try_extract_max_count = 15
    http_conn_timeout = 30  #
    qualys_headers = {}
    multi_proc_batch_number = None
    etld_lib_extract_transform_load_distribute.extract_qualys(
        try_extract_max_count=try_extract_max_count,
        url=url,
        headers=headers,
        payload=payload,
        http_conn_timeout=http_conn_timeout,
        chunk_size_calc=chunk_size_calc,
        output_file=host_xml_file,
        cred_dict=cred_dict,
        qualys_headers_dict=qualys_headers,
        multi_proc_batch_number=multi_proc_batch_number)


def start_msg_host_list_extract():
    etld_lib_functions.logger.info(f"start ")


def end_msg_host_list_extract():
    global url
    global xml_files
    global qualys_headers
    etld_lib_functions.log_file_info(url, 'in')
    for of in xml_files.values():
        of_file = Path(of)
        if of_file.exists():
            etld_lib_functions.log_file_info(of_file)
    for h in qualys_headers.keys():
        etld_lib_functions.logger.info(f"Qualys Header: {h} = {qualys_headers[h]}")

    etld_lib_functions.logger.info(f"end")


def main():
    start_msg_host_list_extract()
    # remove_old_files() Hold off on removing old files for restart.
    host_list_extract()
    end_msg_host_list_extract()


if __name__ == "__main__":
    etld_lib_functions.main(my_logger_prog_name='host_list_extract')
    etld_lib_config.main()
    etld_lib_credentials.main()
    main()






import re
import sqlite3
import time
import multiprocessing
from pathlib import Path
import os
import oschmod
import json
import shelve
import shutil
from qualys_etl.etld_lib import etld_lib_config as etld_lib_config
from qualys_etl.etld_lib import etld_lib_credentials as etld_lib_credentials
from qualys_etl.etld_lib import etld_lib_functions as etld_lib_functions
from qualys_etl.etld_lib import etld_lib_sqlite_tables as etld_lib_sqlite_tables
from qualys_etl.etld_lib import etld_lib_datetime as etld_lib_datetime
import qualys_etl.etld_asset_inventory.asset_inventory_process_00_extract as asset_inventory_extract

global host_list_records
global host_list_records_count
global host_list_get_scope_of_host_ids_sql
global asset_inventory_vm_processed_after
global asset_inventory_multi_proc_batch_size
global asset_inventory_concurrency_limit
global asset_inventory_batch_queue
global asset_inventory_limit_hosts
global spawned_process_info_list
global already_reported_spawned_process_info_status
global qualys_headers_multi_proc_dict
global xml_file_utc_run_datetime


# Create queue of batches to process.
# Each batch will be up to asset_inventory_multi_proc_batch_size.

def remove_old_files():
    if Path(etld_lib_config.asset_inventory_json_dir).name in 'asset_inventory_json_dir':
        os.makedirs(etld_lib_config.asset_inventory_json_dir, exist_ok=True)
        oschmod.set_mode(etld_lib_config.asset_inventory_json_dir, "a+rwx,g-rwx,o-rwx")
    # Remove old json files if they exist in asset_inventory_json_dir
    if Path(etld_lib_config.asset_inventory_json_dir).is_dir():
        if Path(etld_lib_config.asset_inventory_json_dir).name in 'asset_inventory_json_dir':
            count_files = 0
            for f in Path(etld_lib_config.asset_inventory_json_dir).glob('asset_inventory*.json'):
                count_files = count_files + 1
            etld_lib_functions.logger.info(f"Removing {count_files} old json files from dir: "
                                           f"{etld_lib_config.asset_inventory_json_dir}")

            files = Path(etld_lib_config.asset_inventory_json_dir).glob('asset_inventory*.json')
            try:
                for file in files:
                    file.unlink()
            except OSError as e:
                etld_lib_functions.logger.error(f"{e}")
                exit(1)
    try:
        if Path(etld_lib_config.asset_inventory_sqlite_file).is_file():
            etld_lib_functions.logger.info(f"Removing old sqlite file: {etld_lib_config.asset_inventory_sqlite_file}")
            Path(etld_lib_config.asset_inventory_sqlite_file).unlink()
        if Path(etld_lib_config.asset_inventory_shelve_file).is_file():
            etld_lib_functions.logger.info(f"Removing old shelve file: {etld_lib_config.asset_inventory_shelve_file}")
            Path(etld_lib_config.asset_inventory_shelve_file).unlink()
        if Path(etld_lib_config.asset_inventory_csv_file).is_file():
            etld_lib_functions.logger.info(f"Removing old csv file: {etld_lib_config.asset_inventory_csv_file}")
            Path(etld_lib_config.asset_inventory_csv_file).unlink()
    except Exception as e:
        etld_lib_functions.logger.error(f"{e}")
        exit(1)


def start_msg_get_asset_inventory_data():
    etld_lib_functions.logger.info(f"start")


def end_msg_get_asset_inventory_data():
    etld_lib_functions.logger.info(f"host_list get_scope_of_host_ids sql: {host_list_get_scope_of_host_ids_sql}")
    etld_lib_functions.logger.info(f"host_list sqlite file: {etld_lib_config.host_list_sqlite_file}")
    etld_lib_functions.logger.info(f"count host_list host id: {len(host_list_records):,}")
    etld_lib_functions.logger.info(f"end")


def get_next_batch(json_file=None):
    batch_info = {}
    with open(json_file, "r", encoding='utf-8') as read_file:
        with shelve.open(str(etld_lib_config.asset_inventory_temp_shelve_file), flag='n') as shelve_database_temp:
            shelve_database_temp = json.load(read_file)
            if 'hasMore' in shelve_database_temp.keys():
                batch_info = {'responseCode': shelve_database_temp['responseCode'],
                              'count': shelve_database_temp['count'],
                              'hasMore': shelve_database_temp['hasMore'],
                              'lastSeenAssetId': shelve_database_temp['lastSeenAssetId'],
                              }
    return batch_info


def get_asset_inventory_data():
    global qualys_headers_multi_proc_dict
    start_msg_get_asset_inventory_data()
    utc_datetime = etld_lib_datetime.get_utc_date()
    qualys_headers_multi_proc_dict = {}
    batch_info = {'hasMore': '1', 'lastSeenAssetId': 0}
    has_more_records = '1'
    batch_number = 0

    # Get Asset Inventory Count
    batch_number_str = f'batch_{batch_number:06d}'
    asset_inventory_extract.asset_inventory_extract_count(
        asset_last_updated=etld_lib_config.asset_inventory_asset_last_updated,
        last_seen_assetid=str(batch_info['lastSeenAssetId']),
        utc_datetime=utc_datetime,
        batch_number=batch_number_str,
        proc_dict=qualys_headers_multi_proc_dict)

    # Extract Assets
    while has_more_records == '1':
        batch_number = batch_number + 1
        batch_number_str = f'batch_{batch_number:06d}'
        asset_inventory_extract.asset_inventory_extract(
            asset_last_updated=etld_lib_config.asset_inventory_asset_last_updated,
            last_seen_assetid=str(batch_info['lastSeenAssetId']),
            utc_datetime=utc_datetime,
            batch_number=batch_number_str,
            proc_dict=qualys_headers_multi_proc_dict)
        batch_info = get_next_batch(json_file=asset_inventory_extract.json_file)
        etld_lib_functions.logger.info(f"batch_info: {batch_info}")
        if 'hasMore' in batch_info.keys():
            has_more_records = str(batch_info['hasMore'])
        else:
            etld_lib_functions.logger.error("Error downloading records")
            etld_lib_functions.logger.error(f"batch_info: {batch_info}")
            exit(1)
        if batch_number_str in qualys_headers_multi_proc_dict.keys():
            x_ratelimit_remaining = qualys_headers_multi_proc_dict[batch_number_str]['x_ratelimit_remaining']
            if int(x_ratelimit_remaining) < 100:
                # Sleep for 5 minutes and run next.
                etld_lib_functions.logger.warning(f"x_ratelimit_remaining is less than 100. "
                                                  f"Sleeping 5 min.  batch_info: {batch_info}, "
                                                  f"header_info: {qualys_headers_multi_proc_dict[batch_number_str]}")
                time.sleep(300)


def main():
    remove_old_files()
    get_asset_inventory_data()


if __name__ == "__main__":
    etld_lib_functions.main(my_logger_prog_name='asset_inventory_extract_controller')
    etld_lib_config.main()
    etld_lib_credentials.main()
    main()

